@@ -1,5 +1,5 @@
 /* This file is part of Mailfromd.
-   Copyright (C) 2005-2024 Sergey Poznyakoff
+   Copyright (C) 2005-2025 Sergey Poznyakoff
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -36,7 +36,10 @@
 #include "dns.h"
 
 #define DEFAULT_QFLAGS \
-	(adns_qf_quoteok_cname|adns_qf_cname_loose|adns_qf_quoteok_query)
+	(adns_qf_cname_loose | \
+	 adns_qf_quoteok_query | \
+	 adns_qf_quoteok_cname | \
+	 adns_qf_quoteok_anshost)
 
 static mu_debug_handle_t debug_handle;
 static adns_state state;
