@@ -1,5 +1,5 @@
 /* This file is part of Mailfromd.
-   Copyright (C) 2005-2024 Sergey Poznyakoff
+   Copyright (C) 2005-2025 Sergey Poznyakoff
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -841,7 +841,14 @@ callout_mx(struct smtp_io_data *iop, const char *hostname, int *pcount)
 {
 	int i;
 	struct dns_reply reply;
-	mf_status rc, mxstat;
+	mf_status mxstat, status;
+	static int weight[] = {
+		[mf_success] = 0,
+		[mf_not_found] = 100,
+		[mf_failure] = 20,
+		[mf_temp_failure] = 10,
+		[mf_timeout] = 15
+	};
 
 	mxstat = dns_to_mf_status(mx_lookup(hostname, 0, &reply));
 
@@ -851,11 +858,14 @@ callout_mx(struct smtp_io_data *iop, const char *hostname, int *pcount)
 	case mf_success:
 		mu_debug(MF_SOURCE_CALLOUT, MU_DEBUG_TRACE1,
 			 ("Checking MX servers for %s", iop->email));
-		rc = mf_not_found;
+		status = mf_not_found;
 		for (i = 0; i < reply.count; i++) {
-			rc = callout_host(iop, reply.data.str[i]);
-			if (mf_resolved(rc))
+			mf_status rc = callout_host(iop, reply.data.str[i]);
+			if (mf_resolved(rc)) {
+				status = rc;
 				break;
+			} else if (weight[status] > weight[rc])
+				status = rc;
 		}
 		if (pcount)
 			*pcount = reply.count;
@@ -863,10 +873,10 @@ callout_mx(struct smtp_io_data *iop, const char *hostname, int *pcount)
 		break;
 
 	default:
-		rc = mxstat;
+		status = mxstat;
 		break;
 	}
-	return rc;
+	return status;
 }
 
 /* Method "strict". Verifies whether EMAIL is understood either by
