@@ -1,5 +1,5 @@
 /* This file is part of Mailfromd.
-   Copyright (C) 2005-2024 Sergey Poznyakoff
+   Copyright (C) 2005-2025 Sergey Poznyakoff
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -1084,6 +1084,7 @@ code_type_result(NODE *node, struct mu_locus_range const **old_locus)
 {
 	NODE *code, *xcode;
 	
+	mark_locus(node, old_locus);
 	code = result_argptr(node->v.ret.code);
 	xcode = result_argptr(node->v.ret.xcode);
 
@@ -1114,6 +1115,13 @@ code_type_result(NODE *node, struct mu_locus_range const **old_locus)
 	}		
 
 	if (root_node[smtp_state_action]) {
+		code_op(opcode_push);
+		code_immediate(node->locus.beg.mu_line, long);
+
+		struct literal *lit = literal_lookup(node->locus.beg.mu_file);
+		code_op(opcode_push);
+		code_immediate(lit->off, ulong);
+
 		code_result_arg(node->v.ret.message);
 		code_result_arg(xcode);
 		code_result_arg(code);
@@ -3052,14 +3060,15 @@ optimize_type_sedcomp(NODE *node)
 	if (arg->type == node_type_string) {
 		transform_t t = transform_compile(arg->v.literal->text,
 						  node->v.sedcomp.flags);
-		if (!t)
+		if (t) {
+			install_transform(node->v.sedcomp.index, t);
+			node->type = node_type_number;
+			node->v.number = node->v.sedcomp.index;
+			free_node(arg);
+		} else
 			parse_error_locus(&arg->locus,
 					  _("invalid transform string: %s"),
 					  transform_error_string());
-		install_transform(node->v.sedcomp.index, t);
-		node->type = node_type_number;
-		node->v.number = node->v.sedcomp.index;
-		free_node(arg);
 	}
 }
 
