@@ -1,5 +1,5 @@
 /* This file is part of Mailfromd.
-   Copyright (C) 2005-2024 Sergey Poznyakoff
+   Copyright (C) 2005-2025 Sergey Poznyakoff
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -676,14 +676,6 @@ child_start()
 	return 0;
 }
 
-/*
- * Index of the first milter state for which a "prog" is registered.
- * "First" stands here for the state which will be entered first in
- * the normal message processing sequence.  It is used to register the
- * "i" macro to use in that state.
- */
-static enum smtp_state first_used_state = smtp_state_none;
-
 sfsistat
 mlfi_negotiate(SMFICTX *ctx,
 	       unsigned long mta_actions,
@@ -702,13 +694,14 @@ mlfi_negotiate(SMFICTX *ctx,
 		  "filter_actions=%#lx, filter_capa=%#lx",
 		  mta_actions, mta_capa, *filter_actions, *filter_capa));
 
-	if (first_used_state >= smtp_state_first
-	    && first_used_state <= smtp_state_last) {
-		mu_debug(MF_SOURCE_ENGINE, MU_DEBUG_TRACE6,
-			 ("Registering \"i\" macro for state %s",
-			  xstate_to_string(first_used_state)));
-		register_macro(first_used_state, "i");
-	}
+	/*
+	 * First entered sate is connect, because smfilter.xxfi_connect is
+	 * always initialized.  This means "begin" handler (if defined) will
+	 * be run from this state.  Traditionally the "i" macro was
+	 * available in all handlers, therefore it must be registered for
+	 * that state.
+	 */
+	register_macro(smtp_state_connect, "i");
 
 	for (i = 0; i < gacopyz_stage_max; i++) {
 		char *str = get_stage_macro_string(i);
@@ -800,10 +793,6 @@ milter_settimeout(time_t t)
 void
 milter_enable_state(enum smtp_state state)
 {
-	if (state >= smtp_state_first && state <= smtp_state_last
-	    && (first_used_state == smtp_state_none || state < first_used_state))
-		first_used_state = state;
-
 	switch (state) {
 	case smtp_state_connect:
 		smfilter.xxfi_connect = mlfi_connect;
