@@ -1,5 +1,5 @@
 /* This file is part of Mailfromd.
-   Copyright (C) 2005-2024 Sergey Poznyakoff
+   Copyright (C) 2005-2025 Sergey Poznyakoff
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -899,6 +899,13 @@ const char *env_get_macro(eval_environ_t env, const char *symbol);
 
 void mfl_smtp_io_callback(void *data, const char *key, const char *value);
 
+typedef void (*mfd_stack_framer_t) (prog_counter_t,
+				    char const *, char const *,
+				    unsigned, void *);
+void env_stack_trace(eval_environ_t env, mfd_stack_framer_t framer,
+		     void *data);
+
+
 
 /* Runtime functions */
 const char *mailfromd_msgid(SMFICTX *ctx);
