@@ -1,5 +1,5 @@
 /* This file is part of Mailfromd.
-   Copyright (C) 2006-2024 Sergey Poznyakoff
+   Copyright (C) 2006-2025 Sergey Poznyakoff
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -638,15 +638,17 @@ void instr_locus(eval_environ_t env);
 void instr_catch(eval_environ_t env);
 
 static struct mu_locus_point *
-stack_find_locus(eval_environ_t env, prog_counter_t pc, prog_counter_t off, struct mu_locus_point *pt)
+stack_find_locus(eval_environ_t env, prog_counter_t pc, prog_counter_t off,
+		 struct mu_locus_point *pt)
 {
 	int i;
 
 	for (i = 0; i < 32; i++, off++) {
 		if (pc > off && prog[pc - off].c_instr == instr_locus) {
 			pt->mu_file = (char*)(env->dataseg
-					      + mf_cell_c_value(prog[pc - off + 1], ulong));
-			pt->mu_line = mf_cell_c_value(prog[pc - off + 2], uint);
+				 + mf_cell_c_value(prog[pc - off + 1], ulong));
+			pt->mu_line = mf_cell_c_value(prog[pc - off + 2],
+						      uint);
 			return pt;
 		}
 	}
@@ -654,14 +656,16 @@ stack_find_locus(eval_environ_t env, prog_counter_t pc, prog_counter_t off, stru
 }
 
 static void
-stack_trace_entry(eval_environ_t env, prog_counter_t pc)
+stack_trace_entry(eval_environ_t env, prog_counter_t pc,
+		  mfd_stack_framer_t tracer, void *data)
 {
 	char *name;
 	struct mu_locus_point *pt, ptbuf;
 	
 	if (mf_cell_instr(prog[pc-2]) == instr_funcall ||
 	    mf_cell_instr(prog[pc-2]) == instr_builtin) {
-		name = (char*)(env->dataseg + mf_cell_c_value(prog[pc-1], size));
+		name = (char*)(env->dataseg +
+			       mf_cell_c_value(prog[pc-1], size));
 		pc -= 2;
 		pt = stack_find_locus(env, pc, 3, &ptbuf);
 	} else if (mf_cell_instr(prog[pc-2]) == instr_catch) {
@@ -674,31 +678,50 @@ stack_trace_entry(eval_environ_t env, prog_counter_t pc)
 	}
 
 	if (pt)
-		mu_error("%04lu: %s:%u: %s",
-			 (unsigned long) pc,
-			 pt->mu_file, pt->mu_line, name);
+		tracer(pc, name, pt->mu_file, pt->mu_line, data);
 	else
-		mu_error("%04lu: %s",
-			 (unsigned long) pc, name);
+		tracer(pc, name, NULL, 0, data);
 }
 
 void
-runtime_stack_trace(eval_environ_t env)
+env_stack_trace(eval_environ_t env, mfd_stack_framer_t framer, void *data)
 {
 	size_t base;
 
-	mu_error(_("stack trace:"));
-	stack_trace_entry(env, env->pc);
+	stack_trace_entry(env, env->pc, framer, data);
 	for (base = env->base; base < datasize + env->stack_size - 4;
 	     base = mf_c_val(env->dataseg[base + 1], size) + base + 1) {
 		prog_counter_t pc = mf_c_val(env->dataseg[base + 2], ulong);
 		if (pc < 2)
 			break; /*FIXME*/
-		stack_trace_entry(env, pc);
+		stack_trace_entry(env, pc, framer, data);
 		if (mf_cell_instr(prog[pc-2]) == instr_catch) {
-			stack_trace_entry(env, mf_c_val(env->dataseg[base + 5], size));
+			stack_trace_entry(env,
+					  mf_c_val(env->dataseg[base + 5],
+						   size),
+					  framer, data);
 		}
 	}
+}
+
+static void
+print_frame(prog_counter_t pc, char const *func_name,
+	    char const *file_name, unsigned line_num, void *data)
+{
+	if (file_name)
+		mu_error("%04lu: %s:%u: %s",
+			 (unsigned long) pc,
+			 file_name, line_num, func_name);
+	else
+		mu_error("%04lu: %s",
+			 (unsigned long) pc, func_name);
+}
+
+void
+runtime_stack_trace(eval_environ_t env)
+{
+	mu_error(_("stack trace:"));
+	env_stack_trace(env, print_frame, NULL);
 	mu_error(_("stack trace finishes"));
 }
 
@@ -957,6 +980,12 @@ heap_obstack_base(eval_environ_t env)
 	return (void*) env_data_ref(env, env->temp_start);
 }
 
+size_t
+heap_obstack_length(eval_environ_t env)
+{
+	return env->temp_size;
+}
+
 STKVAL *
 env_data_ref(eval_environ_t env, size_t off)
 {
@@ -2331,11 +2360,11 @@ instr_xlat(eval_environ_t env)
 	for (i = 0; i < count; i += 2) {
 		if (mf_c_val(tab[i], long) == mf_c_val(env->reg, long)) {
 			env->reg = tab[i+1];
-			push(env, (STKVAL)0);
+			push(env, (STKVAL)0L);
 			return;
 		}
 	}
-	push(env, (STKVAL)1);
+	push(env, (STKVAL)1L);
 }
 
 void
@@ -2354,11 +2383,11 @@ instr_xlats(eval_environ_t env)
 	for (i = 0; i < count; i += 2) {
 		if (strcmp((char*)(env->dataseg + mf_c_val(tab[i], size)), str) == 0) {
 			env->reg = tab[i+1];
-			push(env, (STKVAL)0);
+			push(env, (STKVAL)0L);
 			return;
 		}
 	}
-	push(env, (STKVAL)1);
+	push(env, (STKVAL)1L);
 }
 
 void
